package com.primeton.eop.openapi.server;

package ..deTemp.1747936846385;

import com.google.common.util.concurrent.RateLimiter;
import com.primeton.eop.authentication.client.AuthenticationClient;
import com.primeton.eop.base.common.constants.CacheConstants;
import com.primeton.eop.base.common.constants.ExceptionCode;
import com.primeton.eop.base.common.constants.GlobalConstants;
import com.primeton.eop.base.common.constants.OpenapiConstants;
import com.primeton.eop.base.common.constants.RedisClusterConstants;
import com.primeton.eop.base.common.constants.RedisStandbyClusterConstants;
import com.primeton.eop.base.common.constants.listener.ContainerResolveLinstener;
import com.primeton.eop.base.common.constants.listener.GlobalResolveListener;
import com.primeton.eop.base.common.constants.listener.RedisResolveListener;
import com.primeton.eop.base.common.constants.listener.Resolver;
import com.primeton.eop.base.common.constants.listener.cache.AfficheCacheResolveListener;
import com.primeton.eop.base.common.constants.listener.cache.ApiCacheResolveListener;
import com.primeton.eop.base.common.constants.listener.cache.AppCacheResolveListener;
import com.primeton.eop.base.common.constants.listener.cache.EsbCacheResolveListener;
import com.primeton.eop.base.common.constants.listener.cache.PacketCacheResolveListener;
import com.primeton.eop.base.common.constants.listener.cache.PartnerCacheResolveListener;
import com.primeton.eop.base.common.constants.listener.cache.RgCacheResolveListener;
import com.primeton.eop.base.common.constants.listener.cache.WhiteIpCacheResolveListener;
import com.primeton.eop.base.common.exception.EOPException;
import com.primeton.eop.base.common.model.Configuration;
import com.primeton.eop.base.common.writer.LogWriterFactory;
import com.primeton.eop.base.redis.pool.ClusterPoolFactory;
import com.primeton.eop.base.redis.pool.StandbyClusterPoolFactory;
import com.primeton.eop.base.redis.zkResolver.RedisChangeResolver;
import com.primeton.eop.base.server.embed.EmbeddedServer;
import com.primeton.eop.base.server.factory.EmveddedServerFactory;
import com.primeton.eop.base.server.listener.ContainerConfigChangeResolver;
import com.primeton.eop.base.zookeeper.app.ZkConfigUtil;
import com.primeton.eop.base.zookeeper.exception.NotRegistedException;
import com.primeton.eop.dbproxy.client.DbProxyClient;
import com.primeton.eop.openapi.biz.util.TypeProcessUtil;
import com.primeton.eop.openapi.engine.internal.Seda;
import com.primeton.eop.openapi.engine.kafkapool.KafkaProducerPool;
import com.primeton.eop.openapi.engine.writetokafka.LogWriterFactoryToKafka;
import com.primeton.eop.openapi.interceptor.pool.resolver.AfficheCacheResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.ApiCacheResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.AppCacheResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.EsbCacheResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.GlobalOpenapiResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.PacketCacheResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.PartnerCacheResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.RgCacheResolver;
import com.primeton.eop.openapi.interceptor.pool.resolver.WhiteIpCacheResolver;
import com.primeton.eop.openapi.interceptor.suyan.SuyanESBEventHandle;
import com.primeton.eop.quota.client.QuotaClient;
import com.primeton.eop.security.client.inner.SecurityInnerClient;
import com.primeton.pub.common.util.ConfSerializeUtil;
import java.util.Map;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public class Bootstrap {
  public static Seda seda = null;
  
  private static String ZK_SERVER = null;
  
  private static String SERVER_HOME = null;
  
  private static String LOG_HOME = null;
  
  private static String SERVER_NAME = null;
  
  private static String SERVER_PORT = null;
  
  private static String INSTANCE_ID = null;
  
  private static ZkConfigUtil zkConfig = null;
  
  private static final Logger logger = LogManager.getLogger(com.primeton.eop.openapi.server.Bootstrap.class);
  
  public static Long max_access_time;
  
  public static RateLimiter limiter;
  
  private static void initConfig() throws NotRegistedException, InstantiationException, IllegalAccessException, EOPException {
    Configuration dbConfiguration = new Configuration();
    dbConfiguration.setAddress(GlobalConstants.SERVERS_CONFIG.getDbProxyAddress());
    DbProxyClient.configuration = dbConfiguration;
    Configuration securityConfiguration = new Configuration();
    securityConfiguration.setAddress(GlobalConstants.SERVERS_CONFIG.getSecurityInnerAddress());
    SecurityInnerClient.configuration = securityConfiguration;
    Configuration quotaConfiguration = new Configuration();
    quotaConfiguration.setAddress(GlobalConstants.SERVERS_CONFIG.getQuotaAddress());
    QuotaClient.configuration = quotaConfiguration;
    Configuration auhenticationConfiguration = new Configuration();
    auhenticationConfiguration.setAddress(GlobalConstants.SERVERS_CONFIG.getAuthenicationAddress());
    AuthenticationClient.configuration = auhenticationConfiguration;
    logger.info("init redis!!");
    zkConfig.register(RedisClusterConstants.class);
    ClusterPoolFactory.init(RedisClusterConstants.REDIS_CLUSTER_CONFIG_POOL);
    zkConfig.register(RedisStandbyClusterConstants.class);
    StandbyClusterPoolFactory.init(RedisStandbyClusterConstants.REDIS_STANDBY_CLUSTER_CONFIG_POOL);
    logger.info("init redis end!!");
    KafkaProducerPool.init();
    logger.info("Init kafkaPool end");
  }
  
  private static void initServer() throws Exception {
    EmbeddedServer jettyServer = EmveddedServerFactory.createJettyServerInstance(Integer.valueOf(SERVER_PORT).intValue(), "/openapi", OpenapiConstants.containerConfig);
    ContainerResolveLinstener.setResolver((Resolver)new ContainerConfigChangeResolver.ServerResolver(jettyServer));
    jettyServer.start();
  }
  
  private static void initLogConfig() throws Exception {
    LogWriterFactory.init("openapi-busi", OpenapiConstants.logConfig);
    LogWriterFactory.init("openapi-time", OpenapiConstants.switchConfig);
    LogWriterFactoryToKafka.init();
  }
  
  private static void initConCurrent() {
    Map<String, String> properties = (new ConfSerializeUtil()).readConfig("concurrent.properties");
    if (properties == null) {
      max_access_time = Long.valueOf(30000L);
      limiter = RateLimiter.create(200.0D);
      SuyanESBEventHandle.suyan_res_max_time = 10000;
    } else {
      max_access_time = Long.valueOf(Long.parseLong(properties.get("max_access_time")));
      Long max_concurrent_num = Long.valueOf(Long.parseLong(properties.get("max_concurrent_num")));
      limiter = RateLimiter.create(max_concurrent_num.longValue());
      SuyanESBEventHandle.suyan_res_max_time = Integer.parseInt(properties.get("suyan_res_max_time"));
    } 
  }
  
  public static void start() throws NotRegistedException, InstantiationException, IllegalAccessException, Exception {
    initConCurrent();
    initBase();
    initZk();
    initConfig();
    initSeda();
    initLogConfig();
    initTypeProcess();
    checkClient();
    initServer();
  }
  
  public static void checkClient() throws EOPException {
    boolean dbResule = DbProxyClient.getInstance().checkConnection();
    if (!dbResule)
      throw new EOPException("DB CONNECTION ERROR", ExceptionCode.SYS_SERVER_NAME_NULL); 
    boolean securityResule = SecurityInnerClient.getInstance().checkConnection();
    if (!securityResule)
      throw new EOPException("SECURITY CONNECTION ERROR", ExceptionCode.SYS_SERVER_NAME_NULL); 
    boolean quotaResule = QuotaClient.getInstance().checkConnection();
    if (!quotaResule)
      throw new EOPException("QUOTA CONNECTION ERROR", ExceptionCode.SYS_SERVER_NAME_NULL); 
    boolean authenticationResule = AuthenticationClient.getInstance().checkConnection();
    if (!authenticationResule)
      throw new EOPException("AUTHENTICATION CONNECTION ERROR", ExceptionCode.SYS_SERVER_NAME_NULL); 
  }
  
  private static void initTypeProcess() throws EOPException {
    try {
      TypeProcessUtil.init();
    } catch (Exception e) {
      throw new EOPException("TypeProcess start error", ExceptionCode.OPENAPI_SEDA_START_ERROR);
    } 
  }
  
  private static void initSeda() throws EOPException {
    try {
      seda = new Seda("seda.xml");
    } catch (EOPException e) {
      throw new EOPException("seda start error", ExceptionCode.OPENAPI_SEDA_START_ERROR);
    } 
  }
  
  private static void initZk() throws NotRegistedException, InstantiationException, IllegalAccessException {
    zkConfig = new ZkConfigUtil(ZK_SERVER);
    zkConfig.register(GlobalConstants.class);
    GlobalResolveListener.setResolver((Resolver)new GlobalOpenapiResolver());
    zkConfig.register(OpenapiConstants.class);
    RedisResolveListener.setResolver((Resolver)new RedisChangeResolver.RedisChangeConnectionResolver());
    zkConfig.register(CacheConstants.class);
    ApiCacheResolveListener.setResolver((Resolver)new ApiCacheResolver());
    AppCacheResolveListener.setResolver((Resolver)new AppCacheResolver());
    EsbCacheResolveListener.setResolver((Resolver)new EsbCacheResolver());
    RgCacheResolveListener.setResolver((Resolver)new RgCacheResolver());
    PacketCacheResolveListener.setResolver((Resolver)new PacketCacheResolver());
    PartnerCacheResolveListener.setResolver((Resolver)new PartnerCacheResolver());
    AfficheCacheResolveListener.setResolver((Resolver)new AfficheCacheResolver());
    WhiteIpCacheResolveListener.setResolver((Resolver)new WhiteIpCacheResolver());
  }
  
  private static void initBase() throws EOPException {
    loadLogHome();
    loadServerHome();
    loadZk();
    loadServerName();
    loadServerPort();
    loadInstanceId();
  }
  
  private static void loadServerPort() throws EOPException {
    SERVER_PORT = System.getProperty("serverport");
    if (SERVER_PORT == null)
      throw new EOPException("SERVER_PORT is null", ExceptionCode.SYS_SERVER_PORT_NULL); 
    logger.info("SERVER_PORT:" + SERVER_PORT);
  }
  
  private static void loadInstanceId() throws EOPException {
    INSTANCE_ID = System.getProperty("instanceid");
    if (INSTANCE_ID == null)
      throw new EOPException("INSTANCE_ID is null", ExceptionCode.SYS_INSTANCE_ID_NULL); 
    logger.info("INSTANCE_ID:" + INSTANCE_ID);
  }
  
  private static void loadServerName() throws EOPException {
    SERVER_NAME = System.getProperty("servername");
    if (SERVER_NAME == null)
      throw new EOPException("SERVER_NAME is null", ExceptionCode.SYS_SERVER_NAME_NULL); 
    logger.info("SERVER_NAME:" + SERVER_NAME);
  }
  
  private static void loadLogHome() throws EOPException {
    LOG_HOME = System.getProperty("logpath");
    if (LOG_HOME == null)
      throw new EOPException("LOG_HOME is null", ExceptionCode.SYS_LOG_HOME_NULL); 
    logger.info("LOG_HOME:" + LOG_HOME);
  }
  
  private static void loadServerHome() throws EOPException {
    SERVER_HOME = System.getProperty("eoppath");
    if (SERVER_HOME == null)
      throw new EOPException("SERVER_HOME is null", ExceptionCode.SYS_SERVER_HOME_NULL); 
    logger.info("SERVER_HOME:" + SERVER_HOME);
  }
  
  private static void loadZk() throws EOPException {
    ZK_SERVER = System.getProperty("zkserver");
    if (ZK_SERVER == null)
      throw new EOPException("ZK_SERVER is null", ExceptionCode.SYS_ZK_SERVER_NULL); 
    logger.info("ZK_SERVER:" + ZK_SERVER);
  }
  
  public static void main(String[] args) {
    try {
      start();
    } catch (Exception e) {
      e.printStackTrace();
      logger.error(e.getMessage());
      logger.fatal("start server error", e);
      System.exit(-1);
    } 
  }
}
